/*
 * snmpwalk.c - send snmp GETNEXT requests to a network entity, walking a
 * subtree.
 *
 */
/**********************************************************************
	Copyright 1988, 1989, 1991, 1992 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

/* Network Tracker
Copyright Ali BASEL
Ali BASEL <alib@sabanciuniv.edu>, <ali@basel.name.tr>
http://www.basel.name.tr
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
*/

#include "tracker.h"

#define MAX_MACS 4096
#define MAX_UPLINK_PORTS 60

oid objid_mib[] = { 1, 3, 6, 1, 2, 1 };
int numprinted=-1;

char mycommunity[255];
oid vlanler[MAX_VLAN_NUMBER];
int VLANLIST=0, ARPLIST=0;
char *host;
struct struct_macler **macler;

int arptable(struct ayr *ayarlar)
{
    int i;
    int router_sayisi=0;
    char **routerlar;
    struct struct_degerler *degerler;
    
    degerler=(struct struct_degerler *)malloc(sizeof(struct struct_degerler)*MAX_DEGER_SAYISI);
    if( !degerler ) {
        syslog(LOG_INFO, "Couldn't allocate memory for degerler MAX_DEGER_SAYISI:%d", MAX_DEGER_SAYISI);
        return 0;
    }
    
    routerlar=router_listesi_al(&router_sayisi, ayarlar);

    syslog(LOG_INFO, "The Total number of routers:%d", router_sayisi);
    for(i=0; i<router_sayisi; i++) syslog(LOG_INFO, "router[%d]:%s", i, routerlar[i]);

    strcpy(mycommunity, ayarlar->SnmpRO); ARPLIST=1;
    for(i=0; i<router_sayisi; i++) {
        host=routerlar[i];
        #ifdef debug
        syslog(LOG_INFO, "ARP listesi icin Router:%s, Community=%s, OID=%s", host, mycommunity, ARP_LIST);
        #endif
        if( host[0]=='#' ) continue; // if #IP, skip
        snmpwalk(ARP_LIST, degerler, ayarlar);
    }

    ARPLIST=0;
    for(i=0; i<router_sayisi; i++) free(routerlar[i]);
    free(routerlar);
    free(degerler);
    return 0;
}



int tracker(struct ayr *ayarlar)
{
    int vlan_sayisi=0, mac_sayisi, toplam_mac_sayisi;
    int i, ii, j=0, jj, sayac_bpn;
    char buf[255], macbuf[255], zaman[100];
    int switch_sayisi=0;
    time_t curtime;
	struct tm *tmb;
    char **switchler;
    static struct struct_degerler *degerler;
    static struct struct_degerler deger;    
    int CISCOMACS_NBR=14, atla=0;
    char CISCOMACS[14][9]={"00:01:42", "00:03:6b", "00:01:9b", "00:b0:64", "00:d0:ba", "00:04:27", "00:30:80", "00:04:c1", "00:08:a3", "00:04:c1", "00:01:96", "00:04:ac", "00:0b:be", "00:0e:38"};

    
	/* If tracking field is not "on", don't do any MAC tracking, and return back */
	if( strcmp(ayarlar->Tracking, "on") ) {
        syslog(LOG_INFO, "Setting Tracking:%s, don't do any MAC tracking !", ayarlar->Tracking);
        return 0;
    }
                            
	/* Tracking isleminin basladigi zamani belirle */
    time(&curtime); tmb=localtime(&curtime);
    sprintf(zaman, "%4d.%02d.%02d-%02d:%02d", tmb->tm_year+1900, tmb->tm_mon+1, tmb->tm_mday, tmb->tm_hour, tmb->tm_min);        
    syslog(LOG_INFO, "LastUpdateStartTime:%s", zaman);
    
    /* Database'den tracking on olan switchlerin listesi alinir ve
	Database'de Stats tablosuna Status=running, LastUpdateTime=zaman degerleri yazilir */
	switchler=switch_listesi_al(&switch_sayisi, ayarlar, zaman);    
    syslog(LOG_INFO, "The number of switches to track MACs:%d", switch_sayisi);
    for(i=0; i<switch_sayisi; i++) syslog(LOG_INFO, "Switch[%d]:%s", i, switchler[i]);

    
    macler=(struct struct_macler **)malloc( MAX_MACS*sizeof(struct struct_macler *) );
    if( !macler ) { syslog(LOG_INFO, "hafiza alinamadi !"); exit(0); }
    for(i=0; i<MAX_MACS; i++) {
        macler[i]=(struct struct_macler *)malloc( MAX_MACS*sizeof(struct struct_macler) );
        if( !macler ) { syslog(LOG_INFO, "hafiza alinamadi !"); exit(0); }
    }    

    degerler=(struct struct_degerler *)malloc(sizeof(struct struct_degerler)*MAX_DEGER_SAYISI);
    if( !degerler ) {
        syslog(LOG_INFO, "Couldn't allocate memory for degerler MAX_DEGER_SAYISI:%d", MAX_DEGER_SAYISI);
        return 0;
    }


    /* Switch'lerde bulunan MAC'ler database'e yazilir */
    for(ii=0; ii<switch_sayisi; ii++) {

        host=switchler[ii]; toplam_mac_sayisi=sayac_bpn=-1;
        if( host[0]=='#' ) continue; // if #IP, skip
        strcpy(mycommunity, ayarlar->SnmpRO);
        #ifdef debug
            syslog(LOG_INFO, "VLAN listesi icin community=%s, OID=%s\n", mycommunity, VLAN_LIST);
        #endif
		
	/* Find active VLANs in the switch and store VLAN numbers in vlanler[0]...vlanler[vlan_sayisi] */
        VLANLIST=1; vlan_sayisi=snmpwalk(VLAN_LIST, degerler, ayarlar); VLANLIST=0;
        #ifdef debug
            syslog(LOG_INFO, "Switch:%s'deki VLAN sayisi=%d\n", host, vlan_sayisi);
        #endif
        if( vlan_sayisi == -1 ) {
            syslog(LOG_INFO, "Switch:%s'deki VLAN sayisi okunamadi !", host);
            continue; /* with the other switch on the list */
        }
        for(i=0; i<vlan_sayisi; i++) {
            #ifdef debug
                syslog(LOG_INFO, "Switch:%s, vlan%d=%lu\n", host, i, vlanler[i]);   
            #endif
        }
        #ifdef debug
            syslog(LOG_INFO, "\nSwitch:%s'deki Her VLAN icin MAC Listesi Alinacak...", host);
        #endif
        for(i=0; i<vlan_sayisi; i++) {
            if( vlanler[i] > 1000 ) { /* Skip VLANs which have VLAN numbers greater than 1000 */
                syslog(LOG_INFO, "Switch:%s, VLAN:%lu 1000'den buyuk oldugu icin pas gec.", host, vlanler[i]);
                continue;
            }
            sprintf(buf, "%s@%lu", ayarlar->SnmpRO, vlanler[i]); strcpy(mycommunity, buf);
            #ifdef debug
                syslog(LOG_INFO, "Switch:%s, VLAN:%lu deki MAC'leri bulmak icin community=%s, OID=%s\n", host, vlanler[i], mycommunity, MAC_LIST);
            #endif
            mac_sayisi=snmpwalk(MAC_LIST, degerler, ayarlar);
            if( mac_sayisi == -1 ) {
                syslog(LOG_INFO, "Switch:%s Vlan:%lu'deki MAC sayisi okunamadi !", host, vlanler[i]);
                continue;
            }
            #ifdef debug
                syslog(LOG_INFO, "Switch:%s, VLAN:%lu deki MAC Sayisi:%d", host, vlanler[i], mac_sayisi);            
            #endif
            for(j=0; j< mac_sayisi; j++) {
                if( !strcmp(degerler[j].val.string, "YOK") ) {
                    #ifdef debug
                        syslog(LOG_INFO, "Switch:%s, VLAN:%lu MAC:YOK", host, vlanler[i]);
                    #endif
                    free( degerler[j].val.string );
                    continue;
                }
                // Remove Cisco's MAC used on the switches
                memcpy( buf, degerler[j].val.string, 6); mac(buf, macbuf); atla=0;
                for(jj=0; jj<CISCOMACS_NBR; jj++) { 
                    if( !strncmp(macbuf, CISCOMACS[jj], 8) ) {
                        #ifdef debug
                            syslog(LOG_INFO, "Switch:%s, vlan:%lu, mac=%s, Cisco MAC pas gec.", host, vlanler[i], macbuf);
                        #endif
                        strcpy(macler[++toplam_mac_sayisi]->mac, "YOK");
                        if( degerler[j].val.string ) free( degerler[j].val.string );
                        atla=1; break;
                    }
                }
                if(atla) continue;                
                //---------------------------------------
                macler[++toplam_mac_sayisi]->vlan=vlanler[i];
                memcpy( macler[toplam_mac_sayisi]->mac, degerler[j].val.string, 6);
                mac(macler[toplam_mac_sayisi]->mac, macbuf);
                for(jj=0; jj<6; jj++)
                    macler[toplam_mac_sayisi]->bpn_index[jj]=degerler[j].last6_oids[jj];
                #ifdef debug
                    syslog(LOG_INFO, "Switch:%s, vlan:%d, mac=%s, BPN:%d.%d.%d.%d.%d.%d-", host, macler[toplam_mac_sayisi]->vlan,macbuf,\
                    macler[toplam_mac_sayisi]->bpn_index[0],\
					macler[toplam_mac_sayisi]->bpn_index[1],\
					macler[toplam_mac_sayisi]->bpn_index[2],\
                    macler[toplam_mac_sayisi]->bpn_index[3],\
					macler[toplam_mac_sayisi]->bpn_index[4],\
					macler[toplam_mac_sayisi]->bpn_index[5]);
                #endif
                if( degerler[j].val.string ) free( degerler[j].val.string );
            }
        } // All MACs in the switch have been found now    

        
        for(i=0; i<toplam_mac_sayisi; i++) {                  
            mac(macler[i]->mac, macbuf);
            if( !strcmp(macler[i]->mac, "YOK") ) {
                #ifdef debug
                    syslog(LOG_INFO, "Switch:%s VLAN:%lu MAC:%s", host, macler[i]->vlan, macbuf);
                    syslog(LOG_INFO, "no MAC veya uplink'deki MAC, pas gec.\n");
                #endif
                continue;
            }
                       
            // Bu MAC'e ait BPN'i bul----------------------
            sprintf(mycommunity, "%s@%d", ayarlar->SnmpRO, macler[i]->vlan);
            sprintf(buf, "%s.%d.%d.%d.%d.%d.%d", BRIDGE_PORT_NUMBERS, macler[i]->bpn_index[0],  macler[i]->bpn_index[1], macler[i]->bpn_index[2],macler[i]->bpn_index[3], macler[i]->bpn_index[4], macler[i]->bpn_index[5]);
            #ifdef debug
                syslog(LOG_INFO, "Switch:%s VLAN:%lu MAC:%s BPN'i bulmak icin community=%s, OID=%s-", host, macler[i]->vlan, macbuf, mycommunity, buf);
            #endif
            if( snmpwalk(buf, (struct struct_degerler *)&deger, ayarlar) == -1 ) {
                strcpy(macler[i]->mac, "YOK");
                #ifdef debug
                    syslog(LOG_INFO, "BPN okunamadi, MAC listeden cikartildi.");
                #endif
                continue;
            }
            if( !strcmp(deger.val.string, "YOK") ) {
                strcpy(macler[i]->mac, "YOK");
                #ifdef debug
                    syslog(LOG_INFO, "Bu MAC expire olmus, BPN'i YOK, MAC de listeden cikartildi.");
                #endif
                continue;
            }
            else {
                #ifdef debug
                    syslog(LOG_INFO, "Bulunan BPN:%d", *deger.val.integer);
                #endif
                // Bu BPN daha once bulunmussa bu bir uplink portudur; bu portlari atla !
                for(jj=0; jj<= toplam_mac_sayisi; jj++) {
                    if( macler[jj]->bridge_port_numbers==*deger.val.integer ) {
                        macler[jj]->bridge_port_numbers=-1; strcpy(macler[jj]->mac, "YOK");
                    }
                }
                macler[i]->bridge_port_numbers=*deger.val.integer;
                if( deger.val.string ) free( deger.val.string );
            }
            //-------------------------------------------

            // Bu MAC'e ait ifindex'i bul----------------------
            strcpy(mycommunity, ayarlar->SnmpRO);            
            sprintf(mycommunity, "%s@%d", ayarlar->SnmpRO, macler[i]->vlan);
            #ifdef debug
                syslog(LOG_INFO, "Switch:%s, VLAN:%d, MAC=%s, ifindex bulmak icin community=%s, OID=%s.%d, bpn=%d", host, macler[i]->vlan, macbuf, mycommunity, IFINDEX, macler[i]->bridge_port_numbers, macler[i]->bridge_port_numbers);
            #endif
            sprintf(buf, "%s.%d", IFINDEX, macler[i]->bridge_port_numbers);
            if( snmpwalk(buf, (struct struct_degerler *)&deger, ayarlar) == -1 ) {
                strcpy(macler[i]->mac, "YOK");
                #ifdef debug
                    syslog(LOG_INFO, "ifindex okunamadi, MAC listeden cikartildi.");
                #endif
                continue;
            }
            macler[i]->ifindex = *deger.val.integer;
            #ifdef debug
                syslog(LOG_INFO, "Bulunan ifindex number=%d\n", macler[i]->ifindex );
            #endif
            if( deger.val.string ) free( deger.val.string );
            //---------------------------------------------
            
            // Bu MAC'e ait CDPindex'i bul------------------
            sprintf(buf, "%s.%d",CDP_INDEX, macler[i]->ifindex);
            #ifdef debug
                syslog(LOG_INFO, "CDP index number bulmak icin community=%s, OID=%s, ifindex=%d\n", mycommunity, buf, macler[i]->ifindex);
            #endif
            if( snmpwalk(buf, (struct struct_degerler *)&deger, ayarlar) == -1 ) {
                strcpy(macler[i]->mac, "YOK");
                #ifdef debug
                    syslog(LOG_INFO, "CDP index okunamadi, MAC listeden cikratildi.");
                #endif
                continue;
            }
            if( strcmp(deger.val.string, "YOK") ) {
                #ifdef debug
                    syslog(LOG_INFO, "CDP is running, this should be an uplink, pas gec !\n");
                #endif
                strcpy( macler[i]->mac, "YOK");
                continue;
            }
            //syslog(LOG_INFO, "Bulunan CDP index number=%lu\n", *deger->val.integer );
            if( deger.val.string ) free( deger.val.string );
            //-------------------------------------------------
            
            // Bu MAC'e ait ifname'i bul----------------------
            sprintf(buf, "%s.%d",INTERFACE_NAMES, macler[i]->ifindex);
            #ifdef debug
                syslog(LOG_INFO, "interface_names bulmak icin community=%s, OID=%s, ifindex=%d\n", mycommunity, buf, macler[i]->ifindex);
            #endif
            if( snmpwalk(buf, (struct struct_degerler *)&deger, ayarlar) == -1 ) {
                strcpy(macler[i]->mac, "YOK");
                #ifdef debug
                    syslog(LOG_INFO, "ifname okunamadi, MAC listeden cikartildi.");
                #endif
                continue;
            }
            strcpy( macler[i]->interface_names, deger.val.string);
            #ifdef debug
                syslog(LOG_INFO, "Bulunan interface_name=%s\n", macler[i]->interface_names );
            #endif
            if( deger.val.string ) free( deger.val.string );
        }


        
        syslog(LOG_INFO, "\n\n\nBulunanlar: Switch %s, Switch'deki MAC Sayisi=%d.\n", host, toplam_mac_sayisi);
        for(i=0; i<toplam_mac_sayisi; i++) {
            if( !strcmp(macler[i]->mac, "YOK") ) {
                #ifdef debug
                syslog(LOG_INFO, "no MAC veya uplink'deki MAC, pas gec.\n");
                #endif
                continue;
            }

            mac(macler[i]->mac, macbuf);
            //#ifdef debug
                syslog(LOG_INFO, "Vlan:%d, mac=%s, bridge_port_numbers=%d, ifindex_numbers=%d, inferface_names=%s.", \
                    macler[i]->vlan, macbuf, macler[i]->bridge_port_numbers, macler[i]->ifindex, macler[i]->interface_names );
            //#endif
        }

        if( toplam_mac_sayisi>0 ) {
            strcpy( macler[toplam_mac_sayisi]->mac, "SON");
            database_yaz( macler, host, ayarlar, zaman );
        }
    }

    for(i=0; i<MAX_MACS; i++) free( macler[i] ); free( macler );    
    for(i=0; i<switch_sayisi; i++) free(switchler[i]);
    free(switchler);
    free(degerler);

    // Router'lardaki ARP tablolarini kullanarak database'e  IP'leri ekle
    arptable(ayarlar);
    #ifdef HAVE_LDAP
        ldap_uid_ekle(ayarlar); // ldap'tan uid'leri ekle
    #endif
    calculate_statistics(ayarlar);
	return 0;
}

int snmpwalk(char *myoid, struct struct_degerler *degerler, struct ayr *ayarlar)
{
    netsnmp_session session, *ss;
    netsnmp_pdu *pdu, *response;
    netsnmp_variable_list *vars;
    int arg;
    oid root[MAX_OID_LEN], name[MAX_OID_LEN];
    size_t rootlen, name_length;
    int count, check, uz=0, i;
    int running;
    int status;
    char *cp=0, buf[255], macbuf[255];
    int exitval = 0;    

    check=1; arg=0; cp=0;
    numprinted=-1; /* Number of OID found */
    
	init_snmp("snmpapp"); /* Initialize the SNMP library */  
        
    /*  Initialize a "session" that defines who we're going to talk to */
    snmp_sess_init( &session ); /* set up defaults */

    session.peername = strdup(host);
    session.version = SNMP_VERSION_2c; /* set the SNMP version number */
    /* set the SNMPv1 community name used for authentication */
    session.community=mycommunity;
    session.community_len = strlen(session.community);

    SOCK_STARTUP;
    ss = snmp_open(&session); /* open an SNMP session  */
    if (ss == NULL) {
        /* diagnose snmp_open errors with the input netsnmp_session pointer  */
        snmp_sess_perror("snmpapp", &session);
        syslog(LOG_INFO, "switch:%s ile snmp session kurulamadi, siradaki ile devam et.", host);
        SOCK_CLEANUP; return -1;
    }

    /* get the initial object and subtree */
    rootlen = MAX_OID_LEN;
    if( snmp_parse_oid(myoid, root, &rootlen) == NULL) {
        snmp_perror(myoid); snmp_close(ss);
        syslog(LOG_INFO, "snmp_parse_oid error:%s siradaki ile devam et.", myoid);        
        //SOCK_CLEANUP;
        return -1;
    }

    /* get first object to start walk */
    memmove(name, root, rootlen * sizeof(oid));
    name_length = rootlen;

    
	running = 1;
	while (running) {
        /* create PDU for GETNEXT request and add object name to request */
        pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
        snmp_add_null_var(pdu, name, name_length);

        /* do the request */
        status = snmp_synch_response(ss, pdu, &response);
        if( status == STAT_SUCCESS ) {
            if( response->errstat == SNMP_ERR_NOERROR ) {
                /* check resulting variables */
                for( vars = response->variables; vars; vars = vars->next_variable) {
                    if( (vars->name_length < rootlen) || (memcmp(root, vars->name, rootlen * sizeof(oid)) != 0)) {
                        /* not part of this subtree  */
                        running = 0; continue;
                    }
                                        
                    if( (vars->type != SNMP_ENDOFMIBVIEW) && (vars->type != SNMP_NOSUCHOBJECT) && (vars->type != SNMP_NOSUCHINSTANCE)) {  /* not an exception value */
                        if( ++numprinted >= MAX_DEGER_SAYISI && !ARPLIST) {
                            syslog(LOG_INFO, "MAX_DEGER_SAYISI exceeded:%d !!!\n\n", numprinted);
                            syslog(LOG_INFO, "Please increase that number in the tracker.h !!\n");
							exit(0);
                        }
                        
                        #ifdef debug
                            print_variable(vars->name, vars->name_length, vars);
                        #endif


                        // Eger ARP icin gelmisse bunu yap, yani 1. adim icin sadece
                        if( ARPLIST) {
                            sprintf(buf, "%lu.%lu.%lu.%lu",\
							vars->name[vars->name_length-4],\
							vars->name[vars->name_length-3],\
                            vars->name[vars->name_length-2],\
							vars->name[vars->name_length-1] );
                        
                            mac(vars->val.string, macbuf);
                            //syslog(LOG_INFO, "Router:%s, bulunan MAC:%s, IP:%s", host, macbuf, buf);
                            ip_ekle(buf, macbuf, ayarlar);// IP --> MAC; IP database'e eklenecek.
                        }
                        
                        // Eger VLAN listesi icin gelmisse bunu yap, yani 1. adim icin sadece
                        else if( VLANLIST ) {
                            #ifdef debug
                                syslog(LOG_INFO, "uz=%d, enson oid=.%lu.\n", vars->name_length, vars->name[vars->name_length-1]);
                            #endif
                            if( numprinted > MAX_VLAN_NUMBER ) {
								syslog(LOG_INFO, "Total active VLAN number exceeded MAX_VLAN_NUMBER:%d\n", MAX_VLAN_NUMBER);
								syslog(LOG_INFO, "Please increase that number in the tracker.h !!\n");
								exit(1);
							}
							vlanler[numprinted]=vars->name[vars->name_length-1];
                        }
                        else { 
                            uz=vars->val_len; // val'in union pointer'indeki data'nin boyutu
                            // Degeri tasiyacak pointer icin yer ayriliyor
                            degerler[numprinted].val.string=(char*)malloc(uz);
                            memcpy( degerler[numprinted].val.string, vars->val.string, uz);
                            degerler[numprinted].val.string[uz]=(char)NULL;
                       
                            #ifdef debug
                                if( vars->type == 2 ) { //integer
                                    syslog(LOG_INFO, "numprinted=%d, type=%d,uz=%d,deger=%lu.\n", numprinted, vars->type, uz, *vars->val.integer);
                                }
                                else {
                                    mac(vars->val.string, macbuf);
                                    syslog(LOG_INFO, "numprinted=%d, type=%d,uz=%d,deger=%s.\n", numprinted, vars->type, uz, macbuf);
                                }
                            #endif
                            for(i=0; i<6; i++)
                                degerler[numprinted].last6_oids[5-i]=vars->name[vars->name_length-i-1];
                        }
                        
                        memmove((char *) name, (char *) vars->name, vars->name_length * sizeof(oid));
                        name_length = vars->name_length;
                    }
                    else {
                        /* an exception value, so stop */
                        running = 0; uz=1;
                        degerler[++numprinted].val.string=(char*)malloc(4);
                        syslog(LOG_INFO, "Aranan deger bulunamadi YOK atandi.");
                        strcpy(degerler[numprinted].val.string, "YOK");
                    }
                }
            }
            else {
                if( running == -1 ) break;
                /* error in response, print it */
                running = 0;
                if (response->errstat == SNMP_ERR_NOSUCHNAME) {
                    printf("End of MIB\n");
                }
                else {
                    printf("Error in packet.\nReason: %s\n", snmp_errstring(response->errstat));
                    if(response->errindex != 0) {
                        printf("Failed object: ");
                        for(count = 1, vars = response->variables; vars && count != response->errindex; vars = vars->next_variable, count++) ;/*EMPTY*/
                        if(vars) fprint_objid(stderr, vars->name, vars->name_length);
                        printf("\n");
                    }
                    exitval = 2;
                }
            }
        }
        else
            if( status == STAT_TIMEOUT ) {
                syslog(LOG_INFO, "Timeout, No Response from %s, FOR THAT community:%s, oid:%s-", session.peername, mycommunity, myoid);
            	running = 0; exitval = 1;
                if( !snmp(host, ayarlar) ) { // snmp'den de cevap gelmediyse yaz
                    // Bir VLAN'de hic MAC yoksa yine timeout oluyor, bunu anlamak icin
                    // bir kontrol daha yapiliyor
                    sprintf(buf, "%s snmp'ye cevap vermedi !", host);
	                syslog(LOG_INFO, "sendmail:from:%s, kime:%s, MTAHost:%s, Subj:%s, Mes:%s\n", ayarlar->FromName, ayarlar->To1, ayarlar->MTAHost, ayarlar->Subject, buf);
	                simple_send_mail(ayarlar->MTAHost, ayarlar->FromName, ayarlar->To1, ayarlar->Subject, buf);
                }
            }
            else {  /* status == STAT_ERROR */
                snmp_sess_perror("snmpwalk", ss);
                running = 0;
                exitval = 1;
            }
        if( response ) snmp_free_pdu(response);
    }
	
    if( numprinted == -1 && status == STAT_SUCCESS ) {
        /* no printed successful results, which may mean we were
        * pointed at an only existing instance.  Attempt a GET, just
        * for get measure. */
        if( !snmp_get_and_print(ss, root, rootlen, &degerler[++numprinted]) )
            numprinted--; // Bir hata olustuysa
    }
    snmp_close(ss);
    if( numprinted > -1 ) numprinted++;
    SOCK_CLEANUP;
    return numprinted;
}

int snmp_get_and_print(netsnmp_session *ss, oid *theoid, size_t theoid_len, struct struct_degerler *deger)
{
    netsnmp_pdu *pdu, *response;
    netsnmp_variable_list *vars;
    int status, uz, sonuc=0, i;

    #ifdef debug
     syslog(LOG_INFO, "snmp_get_and_print cagrildi.");
    #endif
    pdu = snmp_pdu_create(SNMP_MSG_GET);
    snmp_add_null_var(pdu, theoid, theoid_len);

    status = snmp_synch_response(ss, pdu, &response);
    if( status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR ) {
        for(vars = response->variables; vars; vars = vars->next_variable) {
            if( (vars->type != SNMP_ENDOFMIBVIEW) && (vars->type != SNMP_NOSUCHOBJECT) && (vars->type != SNMP_NOSUCHINSTANCE)) {
                /* not an exception value */
                #ifdef debug
                    printf("snmp_get_and_print cagrildi  ve  exception value degil.\n");
                    print_variable(vars->name, vars->name_length, vars);
                #endif
                sonuc=1;
                uz=vars->val_len; // val'in union pointer'indeki data'nin boyutu
                // Degeri tasiyacak pointer icin yer ayriliyor
                deger->val.string=(char*)malloc(uz);
                memcpy( deger->val.string, vars->val.string, uz);
                deger->val.string[uz]=(char)NULL;
                for(i=0; i<6; i++)
                    deger->last6_oids[i]=vars->name[vars->name_length-6-i];
                            
                #ifdef debug
                    if( vars->type == 2 ) { //integer
                        printf("numprinted=%d, type=%d,uz=%d,deger=%lu\n", numprinted, vars->type, uz, *vars->val.integer);
                    }
                    else printf("burasi numprinted=%d, type=%d,uz=%d,deger=%s\n", numprinted, vars->type, uz, vars->val.string);
                #endif
            }
            else { // no such object ve benzeri mesajlar icin
                #ifdef debug
                    printf("snmp_get_and_print cagrildi  ve  exception value !\n");
                #endif
                sonuc=1;
                deger->val.string=(char*)malloc(4);
                strcpy(deger->val.string, "YOK");
            }
        }
    }
    if (response) snmp_free_pdu(response);
    return sonuc;
}
